package com.company.sales.service;

import com.company.sales.entity.Order;
import com.haulmont.cuba.core.EntityManager;
import com.haulmont.cuba.core.Persistence;
import com.haulmont.cuba.core.TypedQuery;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service(OrderService.NAME)
public class OrderServiceBean implements OrderService {

    protected final Persistence persistence;

    public OrderServiceBean(Persistence persistence) {
        this.persistence = persistence;
    }

    @Override
    @Transactional
    public List<Order> getCachedOrders() {
        final TypedQuery<Order> query = createQuery("select e from sales_Order e", Order.class);
        query.setViewName("order-with-customer");
        query.setCacheable(true);
        return query.getResultList();
    }

    @Override
    @Transactional(readOnly = true)
    public List<Order> getNonCachedOrders() {
        final TypedQuery<Order> query = createQuery("select e from sales_Order e", Order.class);
        query.setViewName("order-with-customer");
        query.setCacheable(false);
        return query.getResultList();
    }

    protected <R> TypedQuery<R> createQuery(String qlString, Class<R> resultClass) {
        final TypedQuery<R> query = getEntityManager().createQuery(qlString, resultClass);
        return query;
    }

    protected EntityManager getEntityManager() {
        return persistence.getEntityManager();
    }

}